@@ -55,7 +55,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
           node, obj.get_module(name), name, valself)
     elif isinstance(obj, abstract.Module):
       return self._get_module_attribute(node, obj, name, valself)
-    elif isinstance(obj, abstract.SimpleAbstractValue):
+    elif isinstance(obj, abstract.SimpleValue):
       return self._get_instance_attribute(node, obj, name, valself)
     elif isinstance(obj, abstract.Union):
       if name == "__getitem__":
@@ -73,15 +73,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       else:
         return node, None
     elif isinstance(obj, special_builtins.SuperInstance):
-      if obj.super_obj:
-        cls = obj.super_cls
-        valself = obj.super_obj.to_binding(node)
-        skip = obj.super_cls
-      else:
-        cls = self.vm.convert.super_type
-        skip = None
-      return self._lookup_from_mro_and_handle_descriptors(
-          node, cls, name, valself, skip)
+      return self._get_attribute_from_super_instance(node, obj, name, valself)
     elif isinstance(obj, special_builtins.Super):
       return self.get_attribute(node, self.vm.convert.super_type, name, valself)
     elif isinstance(obj, abstract.BoundFunction):
@@ -89,7 +81,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
     elif isinstance(obj, abstract.TypeParameterInstance):
       param_var = obj.instance.get_instance_type_parameter(obj.name)
       if not param_var.bindings:
-        param_var = obj.param.instantiate(self.vm.root_cfg_node)
+        param_var = obj.param.instantiate(self.vm.root_node)
       results = []
       nodes = []
       for v in param_var.data:
@@ -141,7 +133,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       return node
     elif isinstance(obj, (abstract.StaticMethod, abstract.ClassMethod)):
       return self.set_attribute(node, obj.method, name, value)
-    elif isinstance(obj, abstract.SimpleAbstractValue):
+    elif isinstance(obj, abstract.SimpleValue):
       return self._set_member(node, obj, name, value)
     elif isinstance(obj, abstract.BoundFunction):
       return self.set_attribute(node, obj.underlying, name, value)
@@ -171,11 +163,11 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       # "Any" etc.
       return True
     for baseclass in obj.cls.mro:
-      if baseclass.full_name == "__builtin__.object":
+      if baseclass.full_name == "builtins.object":
         # It's not possible to set an attribute on object itself.
         # (object has __setattr__, but that honors __slots__.)
         continue
-      if (isinstance(baseclass, abstract.SimpleAbstractValue) and
+      if (isinstance(baseclass, abstract.SimpleValue) and
           ("__setattr__" in baseclass or name in baseclass)):
         return True  # This is a programmatic attribute.
       if baseclass.slots is None or name in baseclass.slots:
@@ -224,7 +216,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
 
   def _get_instance_attribute(self, node, obj, name, valself=None):
     """Get an attribute from an instance."""
-    assert isinstance(obj, abstract.SimpleAbstractValue)
+    assert isinstance(obj, abstract.SimpleValue)
     return self._get_attribute(node, obj, obj.cls, name, valself)
 
   def _get_attribute(self, node, obj, cls, name, valself):
@@ -256,7 +248,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       if isinstance(obj, mixin.Class):
         # A class is an instance of its metaclass.
         node, attr = self._lookup_from_mro_and_handle_descriptors(
-            node, obj, name, valself, skip=None)
+            node, obj, name, valself, skip=())
       else:
         node, attr = self._get_member(node, obj, name)
     if attr is None and cls:
@@ -283,6 +275,53 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       attr = self.vm.new_unsolvable(node)
     return node, attr
 
+  def _get_attribute_from_super_instance(
+      self, node, obj: special_builtins.SuperInstance, name, valself):
+    """Get an attribute from a super instance."""
+    # A SuperInstance has `super_cls` and `super_obj` attributes recording the
+    # arguments that super was (explicitly or implicitly) called with. For
+    # example, if the call is `super(Foo, self)`, then super_cls=Foo,
+    # super_obj=self. When the arguments  are omitted, super_cls and super_obj
+    # are inferred from the surrounding context.
+    if obj.super_obj:
+      # When we have a chain of super calls, `starting_cls` is the cls in which
+      # the first call was made, and `current_cls` is the one currently being
+      # processed. E.g., for:
+      #  class Foo(Bar):
+      #    def __init__(self):
+      #      super().__init__()  # line 3
+      #  class Bar(Baz):
+      #    def __init__(self):
+      #      super().__init__()  # line 6
+      # if we're looking up super.__init__ in line 6 as part of analyzing the
+      # super call in line 3, then starting_cls=Foo, current_cls=Bar.
+      if (isinstance(obj.super_obj.cls,
+                     (type(None), abstract.AMBIGUOUS_OR_EMPTY)) or
+          isinstance(obj.super_cls, abstract.AMBIGUOUS_OR_EMPTY)):
+        # Setting starting_cls to the current class when either of them is
+        # ambiguous is technically incorrect but behaves correctly in the common
+        # case of there being only a single super call.
+        starting_cls = obj.super_cls
+      elif obj.super_cls in obj.super_obj.mro:  # super() in a classmethod
+        starting_cls = obj.super_obj
+      else:
+        starting_cls = obj.super_obj.cls
+      current_cls = obj.super_cls
+      valself = obj.super_obj.to_binding(node)
+      # When multiple inheritance is present, the two classes' MROs may differ.
+      # In this case, we want to use the MRO of starting_cls but skip all the
+      # classes up to and including current_cls.
+      skip = set()
+      for parent in starting_cls.mro:
+        skip.add(parent)
+        if parent.full_name == current_cls.full_name:
+          break
+    else:
+      starting_cls = self.vm.convert.super_type
+      skip = ()
+    return self._lookup_from_mro_and_handle_descriptors(
+        node, starting_cls, name, valself, skip)
+
   def _lookup_from_mro_and_handle_descriptors(
       self, node, cls, name, valself, skip):
     attr = self._lookup_from_mro(node, cls, name, valself, skip)
@@ -322,10 +361,9 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
     if (valself and not isinstance(valself.data, abstract.Module) and
         self._computable(name)):
       attr_var = self._lookup_from_mro(node, cls, compute_function, valself,
-                                       skip=self.vm.convert.object_type)
+                                       skip={self.vm.convert.object_type})
       if attr_var and attr_var.bindings:
-        name_var = abstract.AbstractOrConcreteValue(
-            name, self.vm.convert.str_type, self.vm).to_variable(node)
+        name_var = self.vm.convert.constant_to_var(name, node=node)
         return self.vm.call_function(node, attr_var, function.Args((name_var,)))
     return node, None
 
@@ -338,8 +376,8 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
     ret = self.vm.program.NewVariable()
     add_origins = [valself] if valself else []
     for base in cls.mro:
-      # Potentially skip start of MRO, for super()
-      if base is skip:
+      # Potentially skip part of MRO, for super()
+      if base in skip:
         continue
       # When a special attribute is defined on a class buried in the MRO,
       # get_attribute (which calls get_special_attribute) is never called on
@@ -357,7 +395,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
           # we shouldn't bind the function to the class.
           if (not isinstance(value, abstract.PyTDFunction) or
               not isinstance(base, abstract.InterpreterClass)):
-            # See AtomicAbstractValue.property_get for an explanation of the
+            # See BaseValue.property_get for an explanation of the
             # parameters we're passing here.
             value = value.property_get(
                 valself.AssignToNewVariable(node),
@@ -391,7 +429,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
 
   def _get_member(self, node, obj, name):
     """Get a member of an object."""
-    if obj.is_lazy:
+    if isinstance(obj, mixin.LazyMembers):
       obj.load_lazy_attribute(name)
 
     # If we are looking up a member that we can determine is an instance
@@ -455,7 +493,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       return None
 
   def _maybe_load_as_instance_attribute(self, node, obj, name):
-    assert isinstance(obj, abstract.SimpleAbstractValue)
+    assert isinstance(obj, abstract.SimpleValue)
     if not isinstance(obj.cls, mixin.Class):
       return
     for base in obj.cls.mro:
@@ -474,7 +512,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
     """Set a member on an object."""
     assert isinstance(var, cfg.Variable)
 
-    if obj.is_lazy:
+    if isinstance(obj, mixin.LazyMembers):
       obj.load_lazy_attribute(name)
 
     if name == "__class__":
@@ -490,7 +528,7 @@ class AbstractAttributeHandler(utils.VirtualMachineWeakrefMixin):
       # The previous value needs to be loaded at the root node so that
       # (1) it is overwritten by the current value and (2) it is still
       # visible on branches where the current value is not
-      self._maybe_load_as_instance_attribute(self.vm.root_cfg_node, obj, name)
+      self._maybe_load_as_instance_attribute(self.vm.root_node, obj, name)
 
     variable = obj.members.get(name)
     if variable:
